package org.dromara.ygpassport.controller;

import java.util.List;

import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.ygpassport.domain.vo.PhysiqueTestVo;
import org.dromara.ygpassport.domain.bo.PhysiqueTestBo;
import org.dromara.ygpassport.service.IPhysiqueTestService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 学生体质健康测试数据
 *
 * @author Lion Li
 * @date 2024-03-16
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/ygpassport/physiqueTest")
public class PhysiqueTestController extends BaseController {

    private final IPhysiqueTestService physiqueTestService;

    /**
     * 查询学生体质健康测试数据列表
     */
    @SaCheckPermission("ygpassport:physiqueTest:list")
    @GetMapping("/list")
    public TableDataInfo<PhysiqueTestVo> list(PhysiqueTestBo bo, PageQuery pageQuery) {
        return physiqueTestService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出学生体质健康测试数据列表
     */
    @SaCheckPermission("ygpassport:physiqueTest:export")
    @Log(title = "学生体质健康测试数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(PhysiqueTestBo bo, HttpServletResponse response) {
        List<PhysiqueTestVo> list = physiqueTestService.queryList(bo);
        ExcelUtil.exportExcel(list, "学生体质健康测试数据", PhysiqueTestVo.class, response);
    }

    /**
     * 获取学生体质健康测试数据详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("ygpassport:physiqueTest:query")
    @GetMapping("/{id}")
    public R<PhysiqueTestVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(physiqueTestService.queryById(id));
    }

    /**
     * 新增学生体质健康测试数据
     */
    @SaCheckPermission("ygpassport:physiqueTest:add")
    @Log(title = "学生体质健康测试数据", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody PhysiqueTestBo bo) {
        return toAjax(physiqueTestService.insertByBo(bo));
    }

    /**
     * 修改学生体质健康测试数据
     */
    @SaCheckPermission("ygpassport:physiqueTest:edit")
    @Log(title = "学生体质健康测试数据", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody PhysiqueTestBo bo) {
        return toAjax(physiqueTestService.updateByBo(bo));
    }

    /**
     * 删除学生体质健康测试数据
     *
     * @param ids 主键串
     */
    @SaCheckPermission("ygpassport:physiqueTest:remove")
    @Log(title = "学生体质健康测试数据", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(physiqueTestService.deleteWithValidByIds(List.of(ids), true));
    }
}
