package org.dromara.ygpassport.controller;

import java.util.ArrayList;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.toolkit.MPJWrappers;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.dromara.common.excel.core.ExcelResult;
import org.dromara.ygpassport.domain.StudentLeaderExperience;
import org.dromara.ygpassport.domain.StudentList;
import org.dromara.ygpassport.domain.bo.StudentLeaderExperienceBo;
import org.dromara.ygpassport.domain.dto.StudentLeaderExperienceDto;
import org.dromara.ygpassport.listener.StudentLeaderExperienceImportListener;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.ygpassport.domain.vo.StudentLeaderExperienceVo;
import org.dromara.ygpassport.service.IStudentLeaderExperienceService;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 学生干部经历
 *
 * @author Lion Li
 * @date 2024-01-11
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/ygpassport/studentLeaderExperience")
public class StudentLeaderExperienceController extends BaseController {

    private final IStudentLeaderExperienceService studentLeaderExperienceService;

    /**
     * 查询学生干部经历列表 old
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:list")
    @GetMapping("/list")
    public TableDataInfo<StudentLeaderExperienceVo> list(StudentLeaderExperienceBo bo, PageQuery pageQuery) {
        return studentLeaderExperienceService.queryPageList(bo, pageQuery);
    }

    /**
     * 查询学生干部经历列表 new
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:list")
    @GetMapping("/listNew")
    public IPage<StudentLeaderExperienceDto> listNew(PageQuery pageQuery, StudentLeaderExperienceDto studentLeaderExperienceDto) {
        return studentLeaderExperienceService.selectJoinListPage(new Page<>(pageQuery.getPageNum(), pageQuery.getPageSize()),StudentLeaderExperienceDto.class,
            MPJWrappers.<StudentLeaderExperience>lambdaJoin()
                .selectAll(StudentLeaderExperience.class)
                .select(StudentList::getStuName)
                .leftJoin(StudentList.class, StudentList :: getStuNo, StudentLeaderExperience:: getStuNo));
    }

    /**
     * 导出学生干部经历列表
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:export")
    @Log(title = "学生干部经历", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(StudentLeaderExperienceBo bo, HttpServletResponse response) {
        List<StudentLeaderExperienceVo> list = studentLeaderExperienceService.queryList(bo);
        ExcelUtil.exportExcel(list, "学生干部经历", StudentLeaderExperienceVo.class, response);
    }

    /**
     * 导入学生干部经历数据
     *
     * @param file          导入文件
     * @param updateSupport 是否更新已存在数据
     */
    @Log(title = "学生干部经历", businessType = BusinessType.IMPORT)
    @SaCheckPermission("ygpassport:studentLeaderExperience:import")
    @PostMapping(value = "/importStudentLeaderExperienceData", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public R<Void> importData(@RequestPart("file") MultipartFile file, boolean updateSupport) throws Exception {
        StudentLeaderExperienceImportListener listener = new StudentLeaderExperienceImportListener(updateSupport);
        ExcelResult<StudentLeaderExperienceVo> result = ExcelUtil.importExcel(file.getInputStream(), StudentLeaderExperienceVo.class, listener);
        return R.ok(result.getAnalysis());
    }

    /**
     * 获取导入模板
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "学生干部经历信息", StudentLeaderExperienceVo.class, response);
    }

    /**
     * 获取学生干部经历详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:query")
    @GetMapping("/{id}")
    public R<StudentLeaderExperienceVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(studentLeaderExperienceService.queryById(id));
    }

    /**
     * 新增学生干部经历
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:add")
    @Log(title = "学生干部经历", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody StudentLeaderExperienceBo bo) {
        return toAjax(studentLeaderExperienceService.insertByBo(bo));
    }

    /**
     * 修改学生干部经历
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:edit")
    @Log(title = "学生干部经历", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody StudentLeaderExperienceBo bo) {
        return toAjax(studentLeaderExperienceService.updateByBo(bo));
    }

    /**
     * 删除学生干部经历
     *
     * @param ids 主键串
     */
    @SaCheckPermission("ygpassport:studentLeaderExperience:remove")
    @Log(title = "学生干部经历", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(studentLeaderExperienceService.deleteWithValidByIds(List.of(ids), true));
    }
}
