package org.dromara.ygpassport.controller;

import java.util.ArrayList;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.toolkit.MPJWrappers;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.dromara.common.excel.core.ExcelResult;
import org.dromara.ygpassport.domain.StudentList;
import org.dromara.ygpassport.domain.Tutor;
import org.dromara.ygpassport.domain.dto.StuTutorDto;
import org.dromara.ygpassport.listener.TutorImportListener;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.ygpassport.domain.vo.TutorVo;
import org.dromara.ygpassport.domain.bo.TutorBo;
import org.dromara.ygpassport.service.ITutorService;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.springframework.web.multipart.MultipartFile;

/**
 * 导师组活动
 *
 * @author Lion Li
 * @date 2024-01-11
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/ruoyi-ygpassport/tutor")
public class TutorController extends BaseController {

    private final ITutorService tutorService;

    /**
     * 查询导师组活动列表 old
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:list")
    @GetMapping("/list")
    public TableDataInfo<TutorVo> list(TutorBo bo, PageQuery pageQuery) {
        return tutorService.queryPageList(bo, pageQuery);
    }

    /**
     * 查询导师组活动列表 new
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:list")
    @GetMapping("/listNew")
    public IPage<StuTutorDto> listNew(PageQuery pageQuery, StuTutorDto stuTutorDto) {
        return tutorService.selectJoinListPage(new Page<>(pageQuery.getPageNum(), pageQuery.getPageSize()),StuTutorDto.class,
            MPJWrappers.<Tutor>lambdaJoin()
                .selectAll(Tutor.class)
                .select(StudentList::getStuName)
                .leftJoin(StudentList.class, StudentList :: getStuNo, Tutor:: getStuNo));
    }

    /**
     * 导出导师组活动列表
     */
    @SaCheckPermission("ygpassport:tutor:export")
    @Log(title = "导师组活动", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(TutorBo bo, HttpServletResponse response) {
        List<TutorVo> list = tutorService.queryList(bo);
        ExcelUtil.exportExcel(list, "导师组活动", TutorVo.class, response);
    }

    /**
     * 导入导师组活动数据
     *
     * @param file          导入文件
     * @param updateSupport 是否更新已存在数据
     */
    @Log(title = "导师组活动", businessType = BusinessType.IMPORT)
    @SaCheckPermission("ygpassport:tutor:import")
    @PostMapping(value = "/importTutorData", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public R<Void> importData(@RequestPart("file") MultipartFile file, boolean updateSupport) throws Exception {
        TutorImportListener listener = new TutorImportListener(updateSupport);
        ExcelResult<TutorVo> result = ExcelUtil.importExcel(file.getInputStream(), TutorVo.class, listener);
        return R.ok(result.getAnalysis());
    }

    /**
     * 获取导入模板
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response) {
        ExcelUtil.exportExcel(new ArrayList<>(), "导师组信息", TutorVo.class, response);
    }

    /**
     * 获取导师组活动详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:query")
    @GetMapping("/{id}")
    public R<TutorVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(tutorService.queryById(id));
    }

    /**
     * 新增导师组活动
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:add")
    @Log(title = "导师组活动", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody TutorBo bo) {
        return toAjax(tutorService.insertByBo(bo));
    }

    /**
     * 修改导师组活动
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:edit")
    @Log(title = "导师组活动", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody TutorBo bo) {
        return toAjax(tutorService.updateByBo(bo));
    }

    /**
     * 删除导师组活动
     *
     * @param ids 主键串
     */
    @SaCheckPermission("ruoyi-ygpassport:tutor:remove")
    @Log(title = "导师组活动", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(tutorService.deleteWithValidByIds(List.of(ids), true));
    }
}
